"""
Komplexnější úloha na kompozici.
Je generován fraktál typu S. koberec.
Každý element fraktálu sám sebe umí rozdělit na části.
Každá část je opět fraktál a tedy opět sebe sama umí rozdělit.
Dle způsobu dělení dostaneme různé typy fraktálů.

Metoda generuj je rozvinutí výše uvedené rekurze.
"""

from PIL import Image #pro hloupe kresleni
from PIL import ImageDraw # chytre kresleni

class Koberec:
    def __init__(self, x=0, y=0, sirka = 300, vyska = 300):
        self._x = x
        self._y = y
        self.__sirka = sirka
        self.__vyska = vyska
    @property
    def X(self):
        """Vlastnost x-ove souradnice leveho horniho rohu"""
        return self._x

    @property
    def Y(self):
        """Vlastnost y-ove souradnice leveho horniho rohu"""
        return self._y

    @property
    def sirka(self):
        """Vlastnost sirky"""
        return self.__sirka

    @property
    def vyska(self):
        """Vlastnost vysky"""
        return self.__vyska

    def __str__(self):
        return "Souradnice( {0.X},{0.Y} ): Sirka {0.sirka}: Vyska {0.vyska}".format(self)

    def Divide(self):
        data = []
        dx = self.__sirka/3
        dy = self.vyska/3

        data.append(Koberec(self.X, self.Y, dx, dy))
        data.append(Koberec(self.X + dx, self.Y, dx, dy))
        data.append(Koberec(self.X + 2 * dx, self.Y, dx, dy))

        data.append(Koberec(self.X, self.Y + dy, dx, dy))
        data.append(Koberec(self.X + 2 * dx, self.Y + dy, dx, dy))

        data.append(Koberec(self.X, self.Y + 2 * dy, dx, dy))
        data.append(Koberec(self.X + dx, self.Y + 2 * dy, dx, dy))
        data.append(Koberec(self.X + 2 * dx, self.Y + 2 * dy, dx, dy))
        return data



def Generate(prvni,maxiter):
    """
    Model generování fraktálu.
    Dochází k postupnému volání na všechny dílčí komponenty.
    """
    act = [prvni] #dej tam prvni
    potomci = []
    for i in range(maxiter):
        #print("iterace {0}:len {1}".format(i, len(act)))
        while len(act):
            for potomek in act.pop().Divide():
                potomci.append(potomek)
        act, potomci=(potomci, act) #přiřazení dva v jednom

    return act

def Visualise(data:list,im_width:int,im_height:int):
    """
    Kresleni pixel po pixelu.
    """
    img = Image.new("RGB", (im_height,im_height))  # obrazek
    pixels = img.load()  # ziskej pixely
    for item in data:
        for x in range(int(item.X),int(item.X+item.sirka)):
            for y in range(int(item.Y),int( item.Y + item.vyska)):
                pixels[x,y] = (100,100,100)
    img.save("test.png")

def Visualise2(data:list,im_width:int,im_height:int):
    """
    Vykresleni obdelnik po obdelniku.
    """
    img = Image.new("RGB", (im_height,im_height))  # obrazek
    imgdraw=ImageDraw.Draw(img)
    for item in data:
        imgdraw.rectangle(((item.X,item.Y),(item.X+item.sirka,item.Y+item.vyska)),fill='blue')
    img.save("test2.png")


def main():
    root=Koberec(sirka=500,vyska=500)
    data = Generate(root,4) # generuj fraktál
    Visualise(data,500,500)
    Visualise2(data,500,500)

main()
